#
# Attrib.tcl
#

package provide Attribute 0.1;

class Attribute {

  extends Element;

  import Output;
  import Indent;
  import String;
  import Comment;
  import Console;
  import Template;
  import library;

  staticarray template;

  field classifier; # either Class or Interface

  source [file join [library] TmplAttr.tcl];
  source [file join [library] TmplBean.tcl];
  

  method object_type {} {
    set type [$self plainType [$self get_type]];
    switch $type {
      byte    { return "Byte" }
      int     { return "Integer" }
      short   { return "Short" }
      long    { return "Long" }
      boolean { return "Boolean" }
      char    { return "Char" }
      void    { return "Void" }
    }
    return $type;
  }
  
  method true_type {} {
    return [$self parseType [$self get_type]];
  }
  
  method init {{initializer {}}} {
    set classifier $initializer;
  }

  method is {aspect} {
    if { [$classifier is interface] } {
      switch $aspect {
	static    { return 1; } # interface attributes are implicitly static
	final     { return 1; } # interface attributes are implicitly final
      }
    }

    # Final attributes are ones declared read-only
    if { $aspect == "final" } {
      if { [$self hasProperty "final"] } {
	return 1;
      }
      return [expr {[$self get_access]} == {"readonly"}];
    }
    return [$self Element.is $aspect];
  }

  method violations {{mode verbose}} {
    set name [$self get_name];
    set num_errors 0;

    if { [$self is final] && [$self get_initial_value] == "" } {
      $self error "$name: uninitialized final field" $mode;
      incr num_errors;
    }

    return $num_errors;
  }

  method print {} {
    set name [$self get_name];
    set attrib "attrib";
    set bean_description "";
    
    if { [$self hasProperty beanprop] } {
      set attrib "[$self prop beanprop]";
      set bean_description "Java Beans $attrib property.";
    }

    Comment print description "[$self get_description]$bean_description";

    set expander [new Expander];
  
    $expander bind visibility "[$self prop visibility]";
    $expander bind static     "[$self prop static]";
    $expander bind final      "[$self prop final]";
    $expander bind transient  "[$self prop transient]";
    $expander bind volatile   "[$self prop volatile]";
    $expander bind type       "[$self true_type]";
    $expander bind Type       "[$self object_type]";
    $expander bind name       "[$self get_name]";
    $expander bind Name       "[String capitalize [$self get_name]]";
    $expander bind beanprop   "[$self prop beanprop]";

    set initial [$self get_initial_value];
    if { $initial != {} } {
      $expander bind initialvalue " = $initial";
    } else {
      $expander bind initialvalue "";
    }
    
    Output print [$expander expand $template($attrib)];

    delete $expander;
  }
}
