#
# Model.tcl
#

class Model {

  extends Element;

  import ObjectInterface;
  import Console; # XXX

  import sc_message;
  import sc_getActiveModel;

  staticfield self [sc_getActiveModel];

  if { $self == "" } { 
    sc_message "No active model" ok stop;
    return NoActiveModel;
  }

  import $self;

  staticmethod mark {} {
    ObjectInterface mark_selected;

    set packages [$self get_marked_objects "Package"];
    foreach package $packages {
      $package mark_descendants "Package";
    }
    set packages [$self get_marked_objects "Package"];
    foreach package $packages {
      $package mark_children "Class";
      $package mark_children "Interface";
    }
    
    set classes [$self get_marked_objects "Class"];
    set interfaces [$self get_marked_objects "Interface"];

    foreach class $classes {
      new Class $class;
      set derivations [$class get_component_derivations];
      foreach derivation $derivations {
        [$derivation get_component] mark;
      }
      innerclasses $class;
    }

    foreach interface $interfaces {
      new Interface $interface;
      set derivations [$interface get_component_derivations];
      foreach derivation $derivations {
        [$derivation get_component] mark;
      }
    }
  }

  staticmethod unmark {} {
    $self unmark_tree;
  }

  staticmethod print {} {
    #
    # Get list of components to print
    #
    set components [$self get_marked_objects "Component"];
    if { [llength $components] == 0 } { 
      sc_message "No components selected" ok information;
      return NoComponentsSelected;
    }

    $self unmark_tree;

    foreach component $components {
      if { [$component is_marked] } {
        continue;
      }
      $component print;
      $component mark;
    }
  }

  staticmethod innerclasses {class} {
    set all_classes [$self get_descendants "Class"];
    set all_interfaces [$self get_descendants "Interface"];
    
    foreach innerclass $all_classes {
      if { [$class get_name] == [$innerclass get_property "innerclassof"]} {
	new Class $innerclass;
	$innerclass setContainer $class;
	$class addInnerclass $innerclass;
	innerclasses $innerclass;
      }
    }
    
    foreach innerclass $all_interfaces {
      if { [$class get_name] == [$innerclass get_property "innerclassof"]} {
	new Interface $innerclass;
	$innerclass setContainer $class;
	$class addInnerclass $innerclass;
	innerclasses $innerclass;
      }
    }
  }
  
  method get_fqn {} {
    set name  [$self get_name];
    return $name;
  }
}
